@@ -1,29 +1,26 @@
 context("unsuperClass")
-
-library(raster)
+library(terra)
 
 ## Set-up test data
-data(lsat)
 lsatNA <- lsat
 lsatNA[20:40, ] <- NA
-
 lsatNA2 <- lsat
-lsatNA2 <- writeRaster(lsatNA2, rasterTmpFile())
-NAvalue(lsatNA2) <- 20
+lsatNA2 <- terra::writeRaster(lsatNA2, .terraTmpFile())
+values(lsatNA2)[is.na(values(lsatNA2))] <- 20
 
 ## Tiny raster bug caused superClass to fail when predictions were written to .grd file 
 test_that("unsuperClass and NA",{
             for(cm in c(TRUE, FALSE)) {
-                expect_is(sc <- unsuperClass(lsat,  nClasses = 2, clusterMap = cm), "unsuperClass")
+                expect_is(sc <- unsuperClass(lsat, nClasses = 2, clusterMap = cm), "unsuperClass")
                 expect_is(scNA <- unsuperClass(lsatNA,  nClasses = 2, clusterMap = cm), "unsuperClass")
                 expect_true(all(is.na(scNA$map[20:40,])))			
-                expect_is(scNA <- unsuperClass(lsatNA2,  nClasses = 2, filename = rasterTmpFile(), clusterMap = cm), "unsuperClass")
-                expect_equal(minValue(scNA$map), 1)
+                expect_is(scNA <- unsuperClass(lsatNA2,  nClasses = 2, filename = .terraTmpFile(), clusterMap = cm), "unsuperClass")
+                expect_equal(sort(unique(scNA$map[])), c(1,2))
             }
         }) 
 
 test_that("terra inputs",{
-	expect_is(sc <- unsuperClass(rast(lsat),  nClasses = 2), "unsuperClass")
+	expect_is(sc <- unsuperClass(lsat, nClasses = 2), "unsuperClass")
 })
 
 ## kmeans prediction function only
@@ -47,7 +44,7 @@ test_that("kmeans predictions",{
 ## pretty print
 test_that("printing method", {
     skip_on_cran()
-    expect_output(print(unsuperClass(lsat,  nClasses = 2)), "unsuperClass results")
+    expect_output(print(unsuperClass(lsat, nClasses = 2)), "unsuperClass results")
 })
 
 
@@ -55,7 +52,7 @@ test_that("printing method", {
 test_that("kmeans fail detection", {
             skip_on_cran()
             set.seed(1)
-            expect_warning(unsuperClass(lsat, nSamples = ncell(lsat), nStarts = 1, nClasses = 20), "doesn't converge properly")
+            expect_warning(unsuperClass(lsat, nSamples = ncell(lsat), nStarts = 1, nClasses = 30), "doesn't converge properly")
         })
 
 ## Helper for symlink-proof filename checking
@@ -68,12 +65,12 @@ slp_bn <- function(path, tmp = basename(tempdir())) {
 test_that("predict.unSuperClass", {
     skip_on_cran()
     uc <- unsuperClass(lsat, nSamples = ncell(lsat), nClasses = 2)
-    expect_s4_class(pred <- predict(uc, lsat), "RasterLayer")
-    expect_equal(unique(uc$map - pred), 0)
+    expect_s4_class(pred <- predict(uc, lsat), "SpatRaster")
+    expect_equal(sum((uc$map - pred)[]), 0)
     
     tmpFile <- tempfile(fileext = ".grd")    
     pred <- predict(uc, lsat, filename = tmpFile )
     expect_false(inMemory(pred))
-    expect_equal(slp_bn(filename(pred)), slp_bn(tmpFile))
+    expect_equal(basename(sources(pred)), basename(tmpFile))
     file.remove(tmpFile, gsub("grd", "gri", tmpFile))
   })
